
# Fast Dense Spectral-Spatial Convolutional Network Framework for Hyperspectral Images Classification: 
This is the Keras implementation of FDSSC.

## Setup
### **Python 3.5.x**
 
You can download Python 3.5.3 at [here](https://www.python.org/downloads/release/python-353/).

### **Tensorflow-gpu v1.8.0**
 
We use Tensorflow-gpu as our computing backend, and you can also use [theano](http://deeplearning.net/software/theano/install.html) as computing backend. For downloading tensorflow, you can find all the information you need at [here](https://www.tensorflow.org/install/). 
If you have pip, you can simply input the following command:

    pip install --user --upgrade tensorflow-gpu
    
### **CUDA v9.0 and cuDNN v7.0**
To run the tensorflow-gpu, you need a Nvidia GPU card with CUDA Compute Capability 3.0 or higher. 

For our source codes, it would be best that you have a CUDA enabled GPU with you, that would save you a lot of time. But it is still OK to run all the experiments without GPU, at the expense of more patience. For KSC HSI dateset, we used GTX 1080Ti (Compute Capability = 6.1) to train our model within about 450 seconds.  

+ You can download CUDA v9.0 at [here](https://developer.nvidia.com/cuda-90-download-archive). Ensure that you append the relevant Cuda pathnames to the %PATH% environment variable as described in the NVIDIA documentation.

+ You can dowmload cuDNN v7.0 at [here](https://developer.nvidia.com/rdp/cudnn-archive). Note that cuDNN is typically installed in a different location from the other CUDA DLLs. Ensure that you add the directory where you installed the cuDNN DLL to your %PATH% environment variable.

If you have a different version of one of the preceding packages, please change to the specified versions. In particular, the cuDNN version must match exactly: TensorFlow will not load if it cannot find cuDNN64_7.dll. To use a different version of cuDNN, you must build from source.

### **Keras 2.1.6**
To install it and related development package, type:

    pip install --user numpy scipy matplotlib scikit-learn scikit-image
    pip install --user keras
    
### **Model visualization**
The keras.utils.vis_utils provides a function to draw the Keras model (using graphviz) and we used it in our code. It relies on pydot-ng and graphviz. To install it, type:

     pip install pydot-ng & brew install graphviz
   
Or you can delete related code. It doesn't matter with classification results.

## Reproducing the results
1) Run the "Train_FDSSC.py". You need type the name of HSI dataset. Model are saved with a hdf5 format in ./models file.

2) Run the notebook "Evaluate_Model.py", in order to evaluate saved models. You need type the name of HSI dataset and the time series number of model(such as '04\_17\_14\_36').

3) Run the notebook "Get\_Classification\_Maps.py", for creating the clasification map. You also need type the name of HSI dataset and the time series number of model. And you will get the result in .mat format and classification maps.
